<?php

namespace Drupal\external_entities\Plugin\ExternalEntities\StorageClient;

use Drupal\Component\Utility\Random;

/**
 * External entities storage client based on a JSON:API.
 *
 * @ExternalEntityStorageClient(
 *   id = "jsonapi",
 *   label = @Translation("JSON:API"),
 *   description = @Translation("Retrieves external entities from a (Drupal) JSON:API source.")
 * )
 */
class JsonApi extends Rest {

  /**
   * {@inheritdoc}
   */
  public function query(array $parameters = [], array $sorts = [], $start = NULL, $length = NULL) {
    $results = parent::query($parameters, $sorts, $start, $length);

    return $results['data'] ?? $results;
  }

  /**
   * {@inheritdoc}
   */
  public function load($id): mixed {
    $result = parent::load($id);

    // Included contains exposed relationship date for fields in JSON:API.
    if (isset($result['data'], $result['included'])) {
      $result['data']['included'] = $result['included'];
    }

    return $result['data'] ?? $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getListQueryParameters(array $parameters = [], $start = NULL, $length = NULL) {
    // Currently always providing a limit.
    $queryParameters = $this->getPagingQueryParameters($start, $length);

    foreach ($parameters as $parameter) {
      $isCondition = isset($parameter['field']) && isset($parameter['value']);
      if ($isCondition) {
        // We are probably dealing with a filter condition.
        // The structure the filter needs to be the following:
        // @example ?filter[recent][condition][path]=changed&filter[recent][condition][operator]=>&filter[recent][condition][value]=1712670000.
        $randomizer = new Random();
        $randomName = $randomizer->name();
        $queryParameters['filter[' . $randomName . '][condition][path]']= $parameter['field'];
        $queryParameters['filter[' . $randomName . '][condition][operator]']= $parameter['operator'] ?? '=';
        $queryParameters['filter[' . $randomName . '][condition][value]']= $parameter['value'];
        continue;
      }
      // @todo Apply parameter operator.
      $queryParameters[$parameter['field']] = is_array($parameter['value'])
        ? implode(',', $parameter['value'])
        : $parameter['value'];
    }

    if (!empty($this->configuration['parameters']['list'])) {
      $queryParameters += $this->configuration['parameters']['list'];
    }

    return $queryParameters;
  }

  /**
   * {@inheritdoc}
   */
  public function getHttpHeaders(): array {
    $headers = parent::getHttpHeaders();
    // JSON:API required the following Accept header.
    $headers['Accept'] = 'application/vnd.api+json';

    return $headers;
  }

}

